open GtkWindow
open GtkMisc
open GtkBase

let string_of_flag = function
  | `START -> "START"
  | `REDIRECTING -> "REDIRECTING"
  | `TRANSFERRING -> "TRANSFERRING"
  | `NEGOTIATING -> "NEGOTIATING"
  | `STOP -> "STOP"
  | `IS_REQUEST -> "IS_REQUEST"
  | `IS_DOCUMENT -> "IS_DOCUMENT"
  | `IS_NETWORK -> "IS_NETWORK"
  | `IS_WINDOW -> "IS_WINDOW"

  | `FAILED_DNS -> "FAILED_DNS"
  | `FAILED_CONNECT -> "FAILED_CONNECT"
  | `FAILED_TIMEOUT -> "FAILED_TIMEOUT"
  | `FAILED_USERCANCELED -> "FAILED_USERCANCELED"

  | `DEFAULTCHROME -> "DEFAULTCHROME"
  | `WINDOWBORDERSON -> "WINDOWBORDERSON"
  | `WINDOWCLOSEON -> "WINDOWCLOSEON"
  | `WINDOWRESIZEON -> "WINDOWRESIZEON"
  | `MENUBARON -> "MENUBARON"
  | `TOOLBARON -> "TOOLBARON"
  | `LOCATIONBARON -> "LOCATIONBARON"
  | `STATUSBARON -> "STATUSBARON"
  | `PERSONALTOOLBARON -> "PERSONALTOOLBARON"
  | `SCROLLBARSON -> "SCROLLBARSON"
  | `TITLEBARON -> "TITLEBARON"
  | `EXTRACHROMEON -> "EXTRACHROMEON"
  | `ALLCHROME -> "ALLCHROME"
  | `WINDOWRAISED -> "WINDOWRAISED"
  | `WINDOWLOWERED -> "WINDOWLOWERED"
  | `CENTERSCREEN -> "CENTERSCREEN"
  | `DEPENDENT -> "DEPENDENT"
  | `MODAL -> "MODAL"
  | `OPENASDIALOG -> "OPENASDIALOG"
  | `OPENASCHROME -> "OPENASCHROME"
;;

let string_of_flags flags = 
  String.concat " | " (List.map string_of_flag flags)
;;

let new_gmmm = ref (fun _ -> raise (Failure "new_gmmm is not init.ed"))

module Windows = struct
  let windows = ref []

  let rec remove w = function
    | w'::ws when w == w' -> ws
    | w'::ws -> w' :: remove w ws
    | [] -> raise Not_found

  let extinct_callbacks = ref []
  let connect_extinct f = 
    extinct_callbacks := f :: !extinct_callbacks
  let disconnect_extinct f = 
    extinct_callbacks := remove f !extinct_callbacks

  let register (w : GWindow.window) = windows := w :: !windows;;
  let unregister w = 
    windows := remove w !windows;
    if !windows = [] then List.iter (fun f -> f ()) !extinct_callbacks
end

class gmmm chromes = 
  let (chromes : GtkMozembed.chrome_flags list) =
    if List.mem `DEFAULTCHROME chromes then
      [ `WINDOWBORDERSON ; `WINDOWCLOSEON ; `WINDOWRESIZEON ;
	`MENUBARON ; `TOOLBARON ; `LOCATIONBARON ; `STATUSBARON;
	`PERSONALTOOLBARON ; `SCROLLBARSON ; `TITLEBARON ]
    else chromes
  in

  let top = GWindow.window ~title: "gmmm" () in
  (* toplevel init *)
  let _ =
    if List.mem `MODAL chromes then top#set_modal true;
    if not (List.mem `OPENASDIALOG chromes) then
      top#set_default_size 640 480;
    Windows.register top;
    top#misc#connect#destroy (fun () -> Windows.unregister top)
  in

  let vbox = GPack.vbox ~packing: top#add () in

  let toolbar = 
    GButton.toolbar ~packing: (vbox#pack ~expand: false ~fill: false) ()
  in

  let url = GEdit.entry 
	          ~editable: true 
                  ~packing: (vbox#pack ~expand: false 
			               ~fill: false) () 
  in

  let moz = GMozembed.mozembed 
      ~packing: (vbox#pack ~expand: true ~fill: true) () 
  in

  let state = 
    GMisc.label ~justify: `LEFT 
      ~packing: (vbox#pack ~expand: false ~fill: true) ()
  in

  let _ =
    (* url *)
    moz#connect#location (fun () -> url#set_text moz#get_location);
    url#connect#activate (fun () -> moz#load_url url#text); 
    if not (List.mem `LOCATIONBARON chromes) then vbox#remove url#coerce;

    (* tool bars *)
    let back =
      toolbar#insert_button ~text:"Back" ~tooltip:"Go Back" 
	~callback:(fun _ -> if moz#can_go_back then moz#go_back) ()
    in
    let forward =
      toolbar#insert_button ~text:"Forward" ~tooltip:"Go Forward" 
	~callback:(fun _ -> if moz#can_go_forward then moz#go_forward) ()
    in
    let _ = toolbar#insert_space () in
    let reload =
      toolbar#insert_button ~text:"Reload" ~tooltip:"Reload" 
	~callback:(fun _ -> moz#reload `NORMAL) ()
    in
    let _ = toolbar#insert_space () in
    let home =
      toolbar#insert_button ~text:"Home" ~tooltip:"Go to Home" 
	~callback:(fun _ -> ()) ()
    in
    let _ = toolbar#insert_space () in
    let stop =
      toolbar#insert_button ~text:"Stop" ~tooltip:"Stop" 
	~callback:(fun _ -> moz#stop_load) ()
    in

    (* when the location is modified, back and forward may change status *)
    moz#connect#location (fun () -> 
      back#misc#set_sensitive moz#can_go_back;
      forward#misc#set_sensitive moz#can_go_forward);
    if not (List.mem `TOOLBARON chromes) then vbox#remove toolbar#coerce;

    (* stop status *)
    moz#connect#net_start (fun () -> stop#misc#set_sensitive true);
    moz#connect#net_stop (fun () -> stop#misc#set_sensitive false);

    (* status bar *)
    moz#connect#link_message (fun () ->
      state#set_text moz#get_link_message);
    if not (List.mem `STATUSBARON chromes) then vbox#remove state#coerce;

    (* size to *)
    moz#connect#size_to (fun ~width ~height ->
      prerr_endline "sizeto!";
      moz#misc#set_geometry ~width ~height ());

    moz#connect#visibility (fun vis ->
      prerr_endline (Printf.sprintf "visibility %s" 
		       (if vis then "true" else "false")));
      
    (* new_window *)  
    moz#connect#new_window (fun chromes -> 
      let newgmmm = !new_gmmm chromes in
      let moz = (newgmmm :> GMozembed.mozembed) in
      if List.mem `OPENASDIALOG chromes then begin
	prerr_endline "top show will be delayed until visility=true";
	let sidopt = ref None in
	sidopt := Some (moz#connect#visibility (function 
	  | true ->
	      newgmmm#top#show ();
	      begin match !sidopt with
	      | Some sid -> moz#misc#disconnect sid
	      | None -> ()
	      end
	  | false -> ()))
      end else newgmmm#top#show ();
      moz);
  
    (* destroy *)
    moz#connect#destroy_browser (fun () -> top#destroy ())
  in

  object 
    inherit GMozembed.mozembed 
	(Obj.magic moz#as_widget : GtkMozembed.mozembed Gtk.obj)
    method top = top

  end

let _ = new_gmmm := (new gmmm : GtkMozembed.chrome_flags list -> gmmm)

let main () =
  let _ = GMain.Main.init ~setlocale: true () in
  Windows.connect_extinct (fun () -> GMain.Main.quit ());
  let gmmm = !new_gmmm [`DEFAULTCHROME] in
  gmmm#load_url "http://pauillac.inria.fr/";
  gmmm#top#show ();
  GMain.Main.main ()

let () = main ()
