(* $Id: gtkMozembed.ml,v 1.1.1.1 2002/03/18 13:50:35 furuse Exp $ *)

open Gaux
open Gtk
open Tags
open GtkBase

type progress_flags = 
  [ `START | `REDIRECTING | `TRANSFERRING | `NEGOTIATING | `STOP |
    `IS_REQUEST | `IS_DOCUMENT | `IS_NETWORK | `IS_WINDOW ] 

type status_flags = 
  [ `FAILED_DNS | `FAILED_CONNECT | `FAILED_TIMEOUT | `FAILED_USERCANCELED ]

type reload_flags = 
  [ `NORMAL | `BYPASSCACHE | `BYPASSPROXY 
  | `BYPASSPROXYANDCACHE | `CHARSETCHANGE ]

type chrome_flags = 
  [ `DEFAULTCHROME | `WINDOWBORDERSON | `WINDOWCLOSEON | `WINDOWRESIZEON 
  | `MENUBARON | `TOOLBARON | `LOCATIONBARON | `STATUSBARON 
  | `PERSONALTOOLBARON | `SCROLLBARSON | `TITLEBARON | `EXTRACHROMEON
  | `ALLCHROME | `WINDOWRAISED | `WINDOWLOWERED | `CENTERSCREEN
  | `DEPENDENT | `MODAL | `OPENASDIALOG | `OPENASCHROME ]
 

type mozembed = [`widget|`container|`bin|`mozembed]
let cast w : mozembed obj = Object.try_cast w "GtkMozEmbed"
external create : unit -> mozembed obj 
    = "ml_gtk_moz_embed_new"
external set_comp_path : string -> unit 
    = "ml_gtk_moz_embed_set_comp_path"
external load_url : [>`mozembed] obj -> string -> unit 
    = "ml_gtk_moz_embed_load_url"
external stop_load : [>`mozembed] obj -> unit
    = "ml_gtk_moz_embed_stop_load"
external can_go_back : [>`mozembed] obj -> bool 
    = "ml_gtk_moz_embed_can_go_back"
external can_go_forward : [>`mozembed] obj -> bool 
    = "ml_gtk_moz_embed_can_go_forward"
external go_back : [>`mozembed] obj -> unit
    = "ml_gtk_moz_embed_go_back"
external go_forward : [>`mozembed] obj -> unit
    = "ml_gtk_moz_embed_go_forward"
external render_data : [>`mozembed] obj -> string -> int -> string -> string
    = "ml_gtk_moz_embed_render_data"
external open_stream : [>`mozembed] obj -> string -> string -> unit
    = "ml_gtk_moz_embed_open_stream"
external append_data : [>`mozembed] obj -> string -> int -> unit
    = "ml_gtk_moz_embed_append_data"
external close_stream : [>`mozembed] obj -> unit
    = "ml_gtk_moz_embed_close_stream"
external get_link_message : [>`mozembed] obj -> string
    = "ml_gtk_moz_embed_get_link_message"
external get_js_status : [>`mozembed] obj -> string
    = "ml_gtk_moz_embed_get_js_status"
external get_title : [>`mozembed] obj -> string
    = "ml_gtk_moz_embed_get_title"
external get_location : [>`mozembed] obj -> string
    = "ml_gtk_moz_embed_get_location"
external reload : [>`mozembed] obj -> reload_flags -> unit
    = "ml_gtk_moz_embed_reload"
external set_chrome_mask : [>`mozembed] obj -> chrome_flags list -> unit
    = "ml_gtk_moz_embed_set_chrome_mask"
external get_chrome_mask : [>`mozembed] obj -> chrome_flags list
    = "ml_gtk_moz_embed_get_chrome_mask"

module Signals = struct
  open GtkArgv
  open GtkSignal

  let link_message : ([>`mozembed],_) t =
    { name = "link_message"; marshaller = marshal_unit }

  let js_status : ([>`mozembed],_) t =
    { name = "js_status"; marshaller = marshal_unit }

  let location : ([>`mozembed],_) t =
    { name = "location"; marshaller = marshal_unit }

  let title : ([>`mozembed],_) t =
    { name = "title"; marshaller = marshal_unit }

  let progress : ([>`mozembed],current:int -> max:int -> unit) t =
    let marshal f argv = function
      | INT current :: INT max :: _ -> f ~current ~max
      | _ -> invalid_arg "GtkMozembed.Signals.marshal_progress"
    in
    { name = "progress"; marshaller = marshal }

  let progress_all : ([>`mozembed], 
		      string option -> current:int -> max:int -> unit) t =
    let marshal f argv = function
      | STRING uri :: INT current :: INT max :: _ -> f uri ~current ~max
      | _ -> invalid_arg "GtkMozembed.Signals.marshal_progress_all"
    in
    { name = "progress_all"; marshaller = marshal }

  external vallist_progress_flags : int -> progress_flags list 
      = "ml_Vallist_progress_flags"
  external vallist_status_flags : nativeint -> status_flags list 
      = "ml_Vallist_status_flags"

  let net_state : ([>`mozembed], 
		   progress_flags list -> status_flags list -> unit) t =
    let marshal f argv = function
      |	INT state :: INT _ (* status *) :: _ ->
	  (* state is actually nativeint *)
	  let status = get_nativeint argv 1 in
	  f (vallist_progress_flags state) (vallist_status_flags status)
      |	_ -> invalid_arg "GtkMozembed.Signals.marshal_net_state"
    in
    { name = "net_state"; marshaller = marshal }

  let net_state_all : ([>`mozembed], 
		       string option -> 
			 progress_flags list -> status_flags list -> unit) t =
    let marshal f argv = function
      |	STRING uri :: INT state :: INT _ (* status *) :: _ ->
	  (* state is actually nativeint *)
	  let status = get_nativeint argv 2 in
	  f uri (vallist_progress_flags state) (vallist_status_flags status)
      |	_ -> invalid_arg "GtkMozembed.Signals.marshal_net_state_all"
    in
    { name = "net_state_all"; marshaller = marshal }

  let net_start : ([>`mozembed],_) t =
    { name = "net_start"; marshaller = marshal_unit }

  let net_stop : ([>`mozembed],_) t =
    { name = "net_stop"; marshaller = marshal_unit }

  external vallist_flags : nativeint -> chrome_flags list
      = "ml_Vallist_chrome_flags"

  let new_window : ([>`mozembed], chrome_flags list -> mozembed obj) t =
    (* since chromemask is 32bit flags, we cannot use 
       parsed gtkArgv.data. We get nativeint from GtkArgv.t directly *)
    let marshal f argv _ =
      let chromemask = get_nativeint argv 1 in
      let res = f (vallist_flags chromemask) in
      GtkArgv.Arg.set_retloc (GtkArgv.nth argv 0) (`OBJECT(Some res))
    in
    { name = "new_window"; marshaller = marshal }

  let visibility : ([>`mozembed], bool -> unit) t =
    let marshal f argv = function
      |	BOOL b :: _ -> f b
      |	_ -> invalid_arg "GtkMozembed.Signals.marshal_visibility"
    in
    { name = "visibility"; marshaller = marshal }

  let destroy_browser : ([>`mozembed],_) t =
    { name = "destroy_browser"; marshaller = marshal_unit }

  (* cb must return false if the uri should be displayed.
     when true, the display is interrupted: no display. *)
  let open_uri : ([>`mozembed], string option -> bool) t =
    let marshal f argv = function
      | STRING uri :: _ ->
          let result = f uri 
          in set_result argv (`BOOL result)
      | _ -> invalid_arg "GtkMozembed.Signals.marshal_open_uri"
    in
    { name = "open_uri"; marshaller = marshal } 

  let size_to : ([>`mozembed], width: int -> height: int -> unit) t =
    let marshal f argv = function
      | INT width :: INT height :: _ -> f ~width ~height
      | _ -> invalid_arg "GtkMozembed.Signals.marshal_size_to"
    in
    { name = "size_to"; marshaller = marshal }

  (* We know nothing about dom_event yet *)

  let dom_key_down : ([>`mozembed],_) t =
    { name = "dom_key_down"; marshaller = marshal_unit }

  let dom_key_press : ([>`mozembed],_) t =
    { name = "dom_key_press"; marshaller = marshal_unit }

  let dom_key_up : ([>`mozembed],_) t =
    { name = "dom_key_up"; marshaller = marshal_unit }

  let dom_mouse_down : ([>`mozembed],_) t =
    { name = "dom_mouse_down"; marshaller = marshal_unit }

  let dom_mouse_up : ([>`mozembed],_) t =
    { name = "dom_mouse_up"; marshaller = marshal_unit }

  let dom_mouse_click : ([>`mozembed],_) t =
    { name = "dom_mouse_click"; marshaller = marshal_unit }

  let dom_mouse_dbl_click : ([>`mozembed],_) t =
    { name = "dom_mouse_dbl_click"; marshaller = marshal_unit }

  let dom_mouse_over : ([>`mozembed],_) t =
    { name = "dom_mouse_over"; marshaller = marshal_unit }

  let dom_mouse_out : ([>`mozembed],_) t =
    { name = "dom_mouse_out"; marshaller = marshal_unit }

(*
    void ( * security_change)     (GtkMozEmbed *embed, gpointer request,
  				guint state);
    void ( * status_change)       (GtkMozEmbed *embed, gpointer request,
  				gint status, gpointer message);
*)
end
